/**
 * Copyright 2019, Optimizely
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
import { Managed } from './managed';
import { ConversionEvent, ImpressionEvent } from './events';
import { EventDispatcher, EventV1Request } from './eventDispatcher';
import { EventQueue } from './eventQueue';
export declare type ProcessableEvents = ConversionEvent | ImpressionEvent;
export declare type EventDispatchResult = {
    result: boolean;
    event: ProcessableEvents;
};
export declare type EventCallback = (result: EventDispatchResult) => void;
export declare type EventTransformer = (event: ProcessableEvents, projectConfig: any) => Promise<void>;
export declare type EventInterceptor = (event: ProcessableEvents, projectConfig: any) => Promise<boolean>;
export interface EventProcessor extends Managed {
    process(event: ProcessableEvents, projectConfig: any): void;
}
export declare abstract class AbstractEventProcessor implements EventProcessor {
    protected transformers: EventTransformer[];
    protected interceptors: EventInterceptor[];
    protected callbacks: EventCallback[];
    protected dispatcher: EventDispatcher;
    protected queue: EventQueue<ProcessableEvents>;
    constructor({ dispatcher, transformers, interceptors, callbacks, flushInterval, maxQueueSize, }: {
        dispatcher: EventDispatcher;
        transformers?: EventTransformer[];
        interceptors?: EventInterceptor[];
        callbacks?: EventCallback[];
        flushInterval?: number;
        maxQueueSize?: number;
    });
    drainQueue(buffer: ProcessableEvents[]): Promise<any>;
    process(event: ProcessableEvents, projectConfig: any): Promise<void>;
    stop(): Promise<any>;
    start(): void;
    protected abstract groupEvents(events: ProcessableEvents[]): ProcessableEvents[][];
    protected abstract formatEvents(events: ProcessableEvents[]): EventV1Request;
}
